; ---purpose---
; Convert a given date to the number of days
; since 01-01-01. This function Will deliver
; wrong results for dates before 1901-01-01.
; -
; Dates are lists of numbers. Order of year,
; month, day is in ISO format. For example,
; July 23rd 1968 becomes '(#1968 #7 #23).

; ---args---
; DATE - date

; ---keywords---
; DATE-TO-DAYS function, date, time, absolute date, days.

; ---see-also---
; DATE-DIFFERENCE

; ---example---
; (DATE-TO-DAYS '(#2003 #9 #21))

:C LIB/CAAR.L
:C LIB/CAAAR.L
:C LIB/LOGAND.L
:C LIB/ZEROP.L
:C LIB/GREATERP.L
:C LIB/REMAINDER.L
:C LIB/DIFFERENCE.L
:C LIB/PLUS.L
:C LIB/TIMES.L
:C LIB/QUOTIENT.L
:C LIB/NTH.L
:C LIB/INCR.L
:C LIB/DECR.L

; ---code---
(DEFUN DATE-TO-DAYS (DATE)
	(LABEL

	((YEAROF (LAMBDA (X) (CAR X)))
	(MONTHOF (LAMBDA (X) (CADR X)))
	(DAYOF (LAMBDA (X) (CADDR X)))

	(DAYS-PER-MONTH
		'(#0 #31 #59 #90 #120 #151 #181 #212 #243 #273 #304 #334))

	(LEAP-YEAR-P (LAMBDA (YEAR)
		(COND ((ZEROP (REMAINDER YEAR '#400)) *T*)
				((ZEROP (REMAINDER YEAR '#100)) *F*)
				(T (ZEROP (REMAINDER YEAR '#4))))))

	(YEAR-TO-DAYS (LAMBDA (YEAR)
		(DIFFERENCE (PLUS (PLUS (TIMES YEAR '#365)
				(QUOTIENT YEAR '#4))
				(QUOTIENT YEAR '#400))
			(QUOTIENT YEAR '#100))))

	(MONTH-TO-DAYS (LAMBDA (YEAR MONTH)
		(COND ((LOGAND (LEAP-YEAR-P YEAR) (GREATERP MONTH '#2))
				(INCR (CAR (NTH DAYS-PER-MONTH MONTH))))
			(T (CAR (NTH DAYS-PER-MONTH MONTH)))))))

	(PLUS (YEAR-TO-DAYS (DECR (YEAROF DATE)))
		(PLUS (MONTH-TO-DAYS (YEAROF DATE) (MONTHOF DATE))
			(DAYOF DATE)))))
