; ---purpose---
; Check if two decimal integer numbers are in
; ascending order. Return *T*, if A<B and otherwise *F*.
; Numbers are represented by lists of decimal digits.

; ---args---
; A - list of digits
; B - list of digits

; ---keywords---
; ILESSP function, less than, ordering, relation
; comparison, predicate, integer

; ---see-also---
; DIGITS, LESSP, IGREATERP, EQUAL

; ---example---
; (ILESSP '#-123 '#0)

:C LIB/LESSP.L
:C LIB/INTEGER.L

; ---code---
(DEFUN ILESSP (A B)
	(LABEL

	; Check if A has less members than B
	((SHORTER (LAMBDA (A B)
		(COND ((NULL A)
				(COND ((NULL B) *F*)
					(T *T*)))
			((NULL B) *F*)
			(T (SHORTER (CDR A) (CDR B))))))

	; Check if A has more members than B
	(LONGER (LAMBDA (A B) (SHORTER B A)))

	; Handle signs and lengths
	(COMPARE (LAMBDA (A B) (COND ((EQ (CAR A) '-)
			; A<0, B>=0 => A<B
			(COND ((EQ (CAR B) '+) *T*)
				; A<0, B<0, A shorter than B => A>B
				((SHORTER A B) *F*)
				; A<0, B<0, A longer than B => A<B
				((LONGER A B) *T*)
				; last resort: compare digits
				(T (LESSP (CDR B) (CDR A)))))
		; A>=0, B<0 => A>B
		((EQ (CAR B) '-) *F*)
		; A>=0, B>=0, A shorter than B => A<B
		((SHORTER A B) *T*)
		; A>=0, B>=0, A longer than B => A>B
		((LONGER A B) *F*)
		; last resort: compare digits
		(T (LESSP (CDR A) (CDR B)))))))

	(COMPARE (INTEGER A) (INTEGER B))))
