; ---purpose---
; Insert a member into an ordered list.
; The predicate P should be asymmetric
; and evaluate to T for each two
; subsequent members of A.

; ---args---
; X - member to insert
; A - ordered list
; P - predicate defining the order of A

; ---keywords---
; INSERT function, sorting, insertion sort
; ordering, lists, ordered lists

; ---see-also---
; REMOVE, SORT, ORDEREDP

; ---example---
; :L LIB/LESSP.L
; (INSERT '#4 '(#3 #5) LESSP)

:C LIB/APPEND.L
:C LIB/REVERSE.L

; ---model---
; (DEFUN INSERT (X A P)
;	(COND ((NULL A) (CONS X NIL))
;		((P X (CAR A)) (CONS X A))
;		(T (CONS (CAR A) (INSERT X (CDR A) P)))))

; ---code---
(DEFUN INSERT (X A P)
	(LABEL ((INS (LAMBDA (A R)
		(COND ((NULL A) (REVERSE (CONS X R)))
			((P X (CAR A)) (APPEND (REVERSE (CONS X R)) A))
			(T (INS (CDR A) (CONS (CAR A) R)))))))
	(INS A NIL)))
