; ---purpose---
; Add two natural decimal numbers. Numbers are
; represented by lists of digits.

; ---args---
; A - list of digits
; B - list of digits

; ---keywords---
; PLUS function, plus, sum, addition, arithmetics, natural

; ---see-also---
; DIGITS, IPLUS, DIFFERENCE, DIVIDE, QUOTIENT, REMAINDER, TIMES

; ---example---
; (PLUS '#5 '#7)

(DEFINE PLUS T)

:C LIB/DIGITS.L
:C LIB/REVERSE.L
:C LIB/PRED.L
:C LIB/SUCC.L

; ---code---
(DEFUN PLUS (A B)
	(LABEL

	; Compute the sum of two digits and a carry flag.
	; X Y - digits
	; CI CO - carry in, out
	; RESULT - (sum . carry out)
	((DSUM (LAMBDA (X Y CI CO)
		(COND ((EQ X 0)
			(COND ((EQ CI 1) (DSUM 1 Y 0 CO))	; add carry flag
				(T (CONS Y CO))))
			((NULL (SUCC Y))					; handle overflow
				(DSUM (PRED X) 0 CI 1))
			(T (DSUM (PRED X) (SUCC Y) CI CO)))))

	; Add two lists of decimal digits forming reverse
	; decimal numbers. The result is in normal order.
	; A B - reverse lists of digits
	; C - carry flag
	; R - result (normal order)
	(ADD (LAMBDA (A B C R)
		(COND ((NULL A)
				(COND ((NULL B)			; A=() ; B=()
						(COND ((EQ C 0) R)		; carry not set? return R
							(T (CONS 1 R))))	; else prepend overflow
			(T (ADD NIL (CDR B)			; B\=(): keep adding leading zeroes
				(CDR (DSUM 0 (CAR B) C 0))
				(CONS (CAR (DSUM 0 (CAR B) C 0)) R)))))
		((NULL B)
			(ADD (CDR A) NIL			; B=() ; A\=()
				(CDR (DSUM (CAR A) 0 C 0))
				(CONS (CAR (DSUM (CAR A) 0 C 0)) R)))
		(T (ADD (CDR A) (CDR B)			; default: advance to next two digits
			(CDR (DSUM (CAR A) (CAR B) C 0))
			(CONS (CAR (DSUM (CAR A) (CAR B) C 0)) R)))))))

	(ADD (REVERSE A) (REVERSE B) 0 NIL)))
