; ---purpose---
; Sort a list using the Quicksort algorithm.

; ---args---
; A - list to sort
; P - predicate defining the desired order

; ---keywords---
; QUICKSORT function, quick sort, ordering, sorting, example

; ---example---
; :L LIB/LESSP.L
; (QUICKSORT '(#5 #9 #1 #7 #3 #0 #2 #4 #8 #6) LESSP)

:C LIB/APPEND.L
:C LIB/CAAR.L
;C LIB/CADR.L
;C LIB/CDDR.L
:C LIB/LOGNOT.L
:C LIB/ORDEREDP.L

; ---code---
(DEFUN QUICKSORT (A P)
	(LABEL

	; Has X at least 3 arguments?
	((CDDRP (LAMBDA (X)
		(COND ((NULL (CDR X)) *F*)
			((NULL (CDDR X)) *F*)
			(T *T*))))

	; Find X mem A for which an Y mem A exists so that P(Y,X) = *T*
	; (in other words: find a non-minimum member of A).
	(NOT-LEAST (LAMBDA (A X)
		(COND ((NULL A) NIL)
			((P (CAR A) X) X)
			(T (NOT-LEAST (CDR A) X)))))

	; Pick the value in the middle of X.
	; Assume RX = (REVERSE X)
	(MIDDLE (LAMBDA (X RX)
		(COND ((EQ (CAR X) (CAR RX)) (CAR X))
			((EQ (CAR X) (CADR RX)) (CAR X))
			(T (MIDDLE (CDR X) (CDR RX))))))

	; Chose a median for partitioning the list.
	(MEDIAN (LAMBDA (X)
		(LABEL ((NLEAST (NOT-LEAST X (CAR X)))
			(MID (MIDDLE X (REVERSE X))))
			(COND ((P MID NLEAST) NLEAST) (T MID)))))

	; Divide the list A into two partitions A1 and A2 where
	;  P(x,M) = *T* for each x mem A1
	;  P(x,M) = *F* for each x mem A2
	; Return (A1 A2)
	(PART (LAMBDA (A M A1 A2)
		(COND ((NULL A)
				(LIST A1 A2))
			((P (CAR A) M)
				(PART (CDR A) M (CONS (CAR A) A1) A2))
			(T
				(PART (CDR A) M A1 (CONS (CAR A) A2))))))

	; Quicksort two partitions and append them.
	; Used by QSORT to avoid partitioning the set twice.
	(PSORT (LAMBDA (PLIST)
		(APPEND (QSORT (CAR PLIST)) (QSORT (CADR PLIST)))))

	; Quicksort A. Desired order defined by P.
	(QSORT (LAMBDA (A)
		(COND ((NULL A) NIL)	; the empty list is trivial
			((NULL (CDR A)) A)	; so is the one-element list
			((LOGNOT (CDDRP A))	; if there are exactly two members...
				(COND ((P (CAR A) (CADR A)) A)		; ...swap if necessary
					(T (LIST (CADR A) (CAR A)))))
			; if A is ordered, return it
			((ORDEREDP A P) A)
			; otherwise, partition A and PSORT the partitions
			(T (PSORT (PART A (MEDIAN A) NIL NIL)))))))

	(QSORT A)))
