; ---purpose---
; Substitute each occurance of a given S-expression X
; in an S-expr X with another s-expr Y.

; ---args---
; X - expression to replace Y with
; Y - expression to find
; Z - expression to process

; ---keywords---
; SUBST function, substitution, substitute, lists

; ---see-also---
; MEMBER, EQUAL, SUBLIS

; ---example---
; (SUBST 'Y1 'X2 '(LAMBDA (X1 X2) (X1 X2)))

:C LIB/EQUAL.L
:C LIB/APPEND.L
:C LIB/REVERSE.L
:C LIB/CAAR.L
;C LIB/CDAR.L
:C LIB/LITATOM.L

; ---model---
; (DEFUN SUBST (X Y Z)
;	(COND ((EQUAL Y Z) X)
;		((ATOM Z) Z)
;		(T (CONS (SUBST X Y (CAR Z)) (SUBST X Y (CDR Z))))))

; ---model---
; ; TAIL-CALL (BUT WAY TOO COMPLEX) VERSION
; ; SEE 'COPY' FOR DETAILS.
; (DEFUN SUBST (X Y Z)
;	(LABEL
;		((PROTECT (LAMBDA (Z) (LIST Z)))
;		(UNPROTECT (LAMBDA (Z) (CAR Z)))
;		(SBS (LAMBDA (Z R S)
;			(COND ((ATOM Z)
;					(COND ((NULL S)
;							(CONS (CADR R) (CAR R)))
;						((ATOM (CAR S))
;							(SBS 'CONS (CONS (CONS (CADR R) (CAR R))
;									(CDDR R))
;								(CDR S)))
;						(T (SBS (PROTECT (CAR (UNPROTECT S)))
;								(CONS (CONS (CADR R) (CAR R)) (CDDR R))
;								(CDR S)))))
;				((EQUAL Y (UNPROTECT Z))
;					(COND ((NULL S) X)
;						((ATOM (CAR S))
;							(SBS 'CONS (CONS X R) (CDR S)))
;						(T (SBS (CAR S) (CONS X R) (CDR S)))))
;				((ATOM (UNPROTECT Z))
;					(COND ((NULL S) (UNPROTECT Z))
;						((ATOM (CAR S))
;							(SBS 'CONS (CONS (UNPROTECT Z) R) (CDR S)))
;						(T (SBS (CAR S) (CONS (UNPROTECT Z) R) (CDR S)))))
;				(T (SBS (PROTECT (CAR (UNPROTECT Z))) R
;						(APPEND (LIST (PROTECT (CDR (UNPROTECT Z))) 'CONS)
;							S)))))))
;		(SBS (PROTECT Z) NIL NIL)))

; ---code---
; LINEAR-RECURSIVE VERSION
; Z MAY NOT CONTAIN ANY IMPROPER LISTS
(DEFUN SUBST (X Y Z)
	(LABEL ((SBS (LAMBDA (Z R)
			(COND ((NULL Z) (REVERSE R))
				((EQUAL Y Z) X)
				((ATOM Z) Z)
				((LOGAND (LITATOM (CAR Z)) (LITATOM (CDR Z)))
					(CONS (SBS (CAR Z) NIL) (SBS (CDR Z) NIL)))
				(T (SBS (CDR Z) (CONS (SBS (CAR Z) NIL) R)))))))
		(SBS Z NIL)))
