; ---purpose---
; Compute the product of two natural decimal numbers.
; Numbers are represented by lists of decimal digits.

; ---args---
; A - list of digits (factor 1)
; B - list of digits (factor 2)

; ---keywords---
; TIMES function, times, multiply, multiplication, product
; arithmetics, natural

; ---see-also---
; DIGITS, ITIMES, PLUS, DIFFERENCE, DIVIDE, QUOTIENT, REMAINDER

; ---example---
; (TIMES '#12 '#5)

(DEFINE TIMES T)

:C LIB/DIGITS.L
:C LIB/PLUS.L
:C LIB/PRED.L
:C LIB/APPEND.L
:C LIB/REVERSE.L
:C LIB/ZEROP.L

; ---code---
(DEFUN TIMES (A B)
	(LABEL

	; X*10 where X=/=0
	((X10 (LAMBDA (X) (APPEND X '#0)))

	; Add A to R B times.
	; A,R are numbers, B is a decimal digit.
	(ADDN (LAMBDA (A B R)
		(COND ((EQ B 0) R)
			(T (ADDN A (PRED B) (PLUS A R))))))

	; R=A*B
	; B is in reverse order.
	(TMS (LAMBDA (A B R)
		(COND ((NULL B) R)
			(T (TMS (X10 A) (CDR B) (ADDN A (CAR B) R)))))))

	; avoid leading zeroes in result
	(COND ((ZEROP A) '#0)
		(T (TMS A (REVERSE B) '#0)))))
