; ---purpose---
; Generate some entropy by un-sorting a list of
; natural numbers. The order of the list A is
; changed in a hard to predict way by extracting
; the (N mod length[A])'th element from A and
; consing it to B. Each extracted element is
; removed from A. When A is empty, the result
; is in B.

; ---args---
; A - list to unsort
; SEED - first element to arrange, where 0<SEED<=length[A]

; ---keywords---
; UNSORT function, entropy, order, sorting, unsorting, random.

; ---see-also---
; SORT, INSERT, ORDEREDP

; ---example---
; :L LIB/IOTA.L
; (UNSORT (IOTA '#10) '#5)

:C LIB/DIGITS.L
:C LIB/LENGTH.L
:C LIB/DECR.L
:C LIB/ZEROP.L
:C LIB/REMAINDER.L

; ---code---
(DEFUN UNSORT(A SEED)
	(LABEL

		; Extract the n'th element of a list
		((NTH (LAMBDA (A N)
			(COND ((ZEROP N) (CAR A))
				(T (NTH (CDR A) (DECR N))))))

		; Generate a list with the n'th element removed
		(NONTH (LAMBDA (A N R)
			(COND ((ZEROP N) 
					(COND ((NULL A) (REVERSE R))
						(T (APPEND (CDR A) (REVERSE R)))))
				(T (NONTH (CDR A) (DECR N) (CONS (CAR A) R))))))

		; Unsort list A of length K by first extracting element N,
		; keep intermediate result in R
		(USRT (LAMBDA (A N K R)
			(COND ((ZEROP K) (CONS (CAR A) R))
				(T (USRT (NONTH A N NIL) (REMAINDER (CAR A) K)
						(DECR K) (CONS (NTH A N) R)))))))

	(USRT A SEED (DECR (LENGTH A)) NIL)))
