; ---purpose---
; Print the lyrics of "99 bottles of beer on the wall".

; ---keywords---
; BOTTLES, 99 bottles of beer, example programs

; ---example---
; (BOTTLES)

; ---code---
; BOTTLES SONG IN PURE LISP
; COPYRIGHT (C) 2003 NILS M HOLM <NMH@T3X.ORG>
;
; THIS IS PURELY SYMBOLIC LISP.
; THERE ARE NO PRE-DEFINED NUMBERS, NOT EVEN DIGITS.
; THERE ARE NO STRINGS.
; THERE IS NO INPUT/OUTPUT.
; THERE ARE NO SIDE EFFECTS.
;
; THE BELOW EXPRESSION REDUCES TO A LIST CONTAINING
; SUBLISTS. EACH SUBLIST CONTAINS ONE VERSE OF THE SONG.
; EACH VERSE CONSISTS OF FOUR SUBLISTS. EACH SUBLIST OF
; A VERSE CONTAINS ONE SENTENCE.
; NUMBERS ARE REPRESENTED BY LISTS OF DIGITS.
;
; FOR EXAMPLE, THE FIRST VERSE OF THE SONG LOOKS LIKE
; THIS:
;
; (((9 9) BOTTLES OF BEER ON THE WALL)
;  ((9 9) BOTTLES OF BEER)
;  (TAKE ONE DOWN AND PASS IT AROUND)
;  ((9 8) BOTTLES OF BEER ON THE WALL))
;
; REQUIRED SUBROUTINES:
; LABEL, LAMBDA, QUOTE, CONS, CAR, CDR, EQ, NULL, COND
;
; HERE WE GO...

(DEFINE BOTTLES (LAMBDA ()
	(LABEL

	; DEFINE SOME SENTENCES
	((BEER-ON-WALL (QUOTE (BOTTLES OF BEER ON THE WALL)))
	(BEER (QUOTE (BOTTLES OF BEER)))
	(TAKE-DOWN (QUOTE (TAKE ONE DOWN AND PASS IT AROUND)))

	; APPEND TWO LISTS
	(APPEND (LAMBDA (A B)
		(COND ((NULL A) B)
			(T (CONS (CAR A) (APPEND (CDR A) B))))))

	; APPEND FOUR LISTS
	(APPEND4 (LAMBDA (A B C D)
		(APPEND A (APPEND B (APPEND C D)))))

	; FIND THE PREDECESSOR OF A DIGIT, WHERE (PRED (QUOTE 0)) => NIL
	(PRED (LAMBDA (N)
		(COND
			((EQ N (QUOTE 1)) (QUOTE 0))
			((EQ N (QUOTE 2)) (QUOTE 1))
			((EQ N (QUOTE 3)) (QUOTE 2))
			((EQ N (QUOTE 4)) (QUOTE 3))
			((EQ N (QUOTE 5)) (QUOTE 4))
			((EQ N (QUOTE 6)) (QUOTE 5))
			((EQ N (QUOTE 7)) (QUOTE 6))
			((EQ N (QUOTE 8)) (QUOTE 7))
			((EQ N (QUOTE 9)) (QUOTE 8))
			(T NIL))))

	; THE CAR OF THE CDR OF A LIST
	(CADR (LAMBDA (X) (CAR (CDR X))))

	; FORM A ONE-ELEMENT LIST
	(LIST (LAMBDA (X) (CONS X NIL)))

	; DECREMENT A TWO-DIGIT NATURAL NUMBER
	(DECREMENT (LAMBDA (N)
		(COND ((NULL (PRED (CADR N)))
				(CONS (PRED (CAR N)) (LIST (QUOTE 9))))
			(T (CONS (CAR N) (LIST (PRED (CADR N))))))))

	; CHECK IF N EQUALS (QUOTE (0 0))
	(ZEROP (LAMBDA (N)
		(COND ((EQ (CAR N) (QUOTE 0))
				(COND ((EQ (CADR N) (QUOTE 0)) T)
						(T NIL)))
			(T NIL))))

	; FORM A VERSE
	(VERSE (LAMBDA (N)
		(APPEND4
			(LIST (APPEND (LIST N) BEER-ON-WALL))
			(LIST (APPEND (LIST N) BEER))
			(LIST TAKE-DOWN)
			(LIST (APPEND (LIST (DECREMENT N)) BEER-ON-WALL)))))

	; COLLECT THE VERSES IN A TAIL-RECURSIVE FUNCTION
	(COUNT-BOTTLES (LAMBDA (N LYRICS)
		(COND ((ZEROP N) LYRICS)
			(T (COUNT-BOTTLES (DECREMENT N)
				(APPEND LYRICS (LIST (VERSE N)))))))))

	(COUNT-BOTTLES (QUOTE (9 9)) NIL))))

