; ---purpose---
; Re-name variables of lambda expressions.
; Lambda variables are re-name in such a way
; that no conflicts can arise between names
; of nested lambda expressions. After
; LVRENAME-ing an expression, full beta
; reduction can be done using SUBST.
; --
; For example,
; -
; ((LVRENAME '(LAMBDA (X) (LAMBDA (X) X)))
; -
; => (LAMBDA (X:0) (LAMBDA (X:1) X:1))

; ---args---
; X - S-expression

; ---keywords---
; LVRENAME function, lambda abstraction, beta reduction
; lambda variables, lambda variable renaming, example

; ---example---
; (LVRENAME '(LAMBDA (X) (LAMBDA (X) X)))

:L LIB/ASSOC.L
:L LIB/REVERSE.L
:L LIB/APPEND.L
:L LIB/NEQ.L
:L LIB/LOGNOT.L
:L LIB/LISTP.L
:L LIB/CAAR.L
;L LIB/CADR.L
;L LIB/CDDR.L
:L LIB/INCR.L
:L LIB/CAAAR.L
;L LIB/CADDR.L

; ---code---
(DEFUN LVRENAME (X)
	(LABEL

		; Re-name X to first match of NLIST
		((RENAME (LAMBDA (X NLIST)
			(LABEL ((NX (ASSOC X NLIST)))
				(COND ((NULL NX) X) (T (CDR NX))))))

		; Re-name each symbol in L
		(RENAME-LIST (LAMBDA (L NLIST R)
			(COND ((NULL L) (REVERSE R))
				(T (RENAME-LIST (CDR L) NLIST
					(CONS (RENAME (CAR L) NLIST) R))))))

		; Create a binding of the form
		; (NAME . UNIQUE-NAME)
		; Unique names are created by appending -LEVEL
		(MAKE-ASSOC (LAMBDA (NAME LEVEL)
			 (CONS NAME
				(IMPLODE (APPEND (EXPLODE NAME)
							(APPEND '#: LEVEL))))))

		; Create a set of local (unique) names
		; OLD   - NLIST so far
		; NEW   - list fo symbols to add to NLIST
		; LEVEL - level of nested LAMBDA
		(LOCALNAMES (LAMBDA (OLD NEW LEVEL)
			(COND ((NULL NEW) OLD)
				(T (CONS (MAKE-ASSOC (CAR NEW) LEVEL)
					(LOCALNAMES OLD (CDR NEW) LEVEL))))))

		; Check if X is a valid lambda expression
		(LAMBDA-P (LAMBDA (X)
			(COND
				((NEQ (CAR X) 'LAMBDA) *F*)
				((NULL (CDR X)) *F*)
				((LOGNOT (LISTP (CDR X))) *F*)
				((LOGNOT (LISTP (CADR X))) *F*)
				(T (LOGNOT (NULL (CDDR X)))))))

		; Re-name all lambda variables in X.
		; NLIST holds the bindings used to re-name symbols.
		; LEVEL is the current nesting level of LAMBDA.
		(LVR (LAMBDA (X NLIST LEVEL)
			(COND
				((NULL X) NIL)
				((ATOM X) (RENAME X NLIST))
				((LAMBDA-P X)
					(LABEL ((NEW-NLIST (LOCALNAMES NLIST (CADR X) LEVEL)))
					(LIST 'LAMBDA
							(RENAME-LIST (CADR X) NEW-NLIST NIL)
							(LVR (CADDR X) NEW-NLIST (INCR LEVEL)))))
				(T (CONS (LVR (CAR X) NLIST LEVEL)
						(LVR (CDR X) NLIST LEVEL)))))))

		(LVR X NIL '#0)))
