; ---purpose---
; Solve the N-queens problem using backtracking.
; The number of queens (N) is specified as a list
; of digits.
; --
; NOTICE: this program is SLOW. It requires more
; than 1.9 million reductions to solve (QUEENS '#4).

; ---args---
; N - list of digits

; ---keywords---
; QUEENS function, queens, eight queens, N queens, N-queens, example

; ---example---
; (QUEENS '#4)

:L LIB/QUOTIENT.L
:L LIB/REMAINDER.L
:L LIB/EQUAL.L
:L LIB/LESSP.L
:L LIB/DIFFERENCE.L
:L LIB/TIMES.L
:L LIB/PLUS.L
:L LIB/NEQUAL.L
:L LIB/INCR.L
:L LIB/DECR.L

; ---code---
(DEFUN QUEENS (BOARD-SIZE)
	(LABEL

	; Translate square number to column number
	((COLUMN (LAMBDA (X) (QUOTIENT X BOARD-SIZE)))

	; Translate square number to row number
	(ROW (LAMBDA (X) (REMAINDER X BOARD-SIZE)))

	; Can X attack Y horizontally or vertically?
	(CAN-ATTACK-HV-P (LAMBDA (X Y)
		(COND ((EQUAL (ROW X) (ROW Y)) *T*)
			(T (EQUAL (COLUMN X) (COLUMN Y))))))

	; Compute |X-Y|
	(ABS-DIFF (LAMBDA (X Y)
		(COND ((LESSP X Y) (DIFFERENCE Y X))
			(T (DIFFERENCE X Y)))))

	; Can X attack Y diagonally?
	(CAN-ATTACK-DIA-P (LAMBDA (X Y)
		(EQUAL (ABS-DIFF (COLUMN X) (COLUMN Y))
			(ABS-DIFF (ROW X) (ROW Y)))))

	; Can X attack Y?
	(CAN-ATTACK-P (LAMBDA (X Y)
		(COND ((CAN-ATTACK-HV-P X Y) T)
			(T (CAN-ATTACK-DIA-P X Y)))))

	; Test whether the square X is safe on a board
	; already containing the pieces listed in B
	(SAFE-PLACE-P (LAMBDA (X B)
		(COND ((NULL B) *T*)
			((CAN-ATTACK-P (CAR B) X) *F*)
			(T (SAFE-PLACE-P X (CDR B))))))

	; Compute the number of the first square
	; of the next column, where Q is any square
	; in the current column
	(NEXT-COLUMN (LAMBDA (Q)
		(TIMES (QUOTIENT (PLUS Q BOARD-SIZE) BOARD-SIZE)
			BOARD-SIZE)))

	; Solve N Queens:
	; Q = next square to check
	; C = current column
	; B = board so far
	(N-QUEENS (LAMBDA (Q C B)
		(COND ((EQUAL C BOARD-SIZE) B)
			((NEQUAL (COLUMN Q) C)
				(COND ((NULL B) NIL)
					(T (N-QUEENS (INCR (CAR B)) (DECR C) (CDR B)))))
			((SAFE-PLACE-P Q B)
				(N-QUEENS (NEXT-COLUMN Q) (INCR C) (CONS Q B)))
			(T
				(N-QUEENS (INCR Q) C B))))))

	(N-QUEENS '#0 '#0 NIL)))
